﻿#include "stdafx.h"
#include "ClientAPI.h"

// insert the IP of your SP-ICE-3
#define CARD_IP "192.168.2.138"

static rlResult markSquarePingPong(rlHandle handle, int* pendingCount, int* listId, int iteration);
static int printLastError(void);


int main()
{
	// establish a connection with the card
	rlHandle handle = rlConnect(CARD_IP, 49374);
	if (handle < 0)
		return printLastError();

	rlResult status;

	// Reset the card to its default settings:
	// This restores the following settings to the values that were last saved on the card:
	// - scanner configuration (such as field size, field correction, etc.)
	// - laser configuration (such as gate/LM timing parameters, power target, etc.)
	// - process variables (such as jump speed, settling times, mark speed, etc.)
	if ((status = rlSystemResetToDefaults(handle)) != rlSUCCESS)
		goto Cleanup;

	// Set the laser power to 40%
	rlLaserConfig lc;
	if ( ( status = rlLaserGetConfig( handle, &lc ) ) != rlSUCCESS ) goto Cleanup;
	lc.PowerScale = 0.4;
	if ( ( status = rlLaserSetConfig( handle, &lc ) ) != rlSUCCESS ) goto Cleanup;

	// Enable the laser so it responds to commands that turn on the laser.
	// Note that this does not fire the laser until e.g. LaserOn or Mark command is executed.
	if ( ( status = rlLaserArmLaser( handle, true ) ) != rlSUCCESS ) goto Cleanup;

	int pendingCount = 0;
	int listId = 0;

	for (int i = 0; i < 10; i++)
	{
		status = markSquarePingPong(handle, &pendingCount, &listId, i);
	}

Cleanup:
	// print any errors
	if (status != rlSUCCESS)
		printLastError();

	// disconnect from the card
	printf("Disconnecting from card...\n");
	if (rlDisconnect(handle) != rlSUCCESS)
		return printLastError();

	printf("Exiting with status = %d\n", status);
	return status;
}

/// by using two lists, the waiting time is minimized
/// as the second list can be prepared while the first list is beeing executed
static rlResult markSquarePingPong(rlHandle handle, int* pendingCount, int* listId, int iteration)
{
	rlResult result;

	// get the scanner's current field size
	rlScannerConfig sc;
	if ((result = rlScannerGetConfig(handle, &sc)) != rlSUCCESS)
		return result;

	double squareSize = 0.05 * sc.FieldSize.X;

	double jumpSpeed = 3.0;
	double markSpeed = 0.2;

	// Create a list locally on the host computer,
	// and fill it with macro-vectors defining a square.
	printf("Preparing list: index=%d...\n", *listId);
	rlCommandListHandle list = rlListAllocate(handle);

	if ((result = rlListAppendPower(list, 65535)) != rlSUCCESS) return result;
	if ((result = rlListAppendJumpSpeed(list, jumpSpeed)) != rlSUCCESS) return result;
	if ((result = rlListAppendMarkSpeed(list, markSpeed)) != rlSUCCESS) return result;
	if ((result = rlListAppendJumpAbs2D(list, -squareSize / 2, -0.45 * sc.FieldSize.X + squareSize * 1.1 * iteration)) != rlSUCCESS) return result;
	if ((result = rlListAppendMarkRel2D(list, squareSize, 0)) != rlSUCCESS) return result;
	if ((result = rlListAppendMarkRel2D(list, 0, squareSize)) != rlSUCCESS) return result;
	if ((result = rlListAppendMarkRel2D(list, -squareSize, 0)) != rlSUCCESS) return result;
	if ((result = rlListAppendMarkRel2D(list, 0, -squareSize)) != rlSUCCESS) return result;
	if ((result = rlListAppendLaserOff(list)) != rlSUCCESS) return result;

	int timeoutMs = 30000;
	bool success = false;
	int maxPendingLists = 2;
	int doneId = 0;

	// only wait for list done, if both lists are busy
	if (*pendingCount >= maxPendingLists)
	{
		printf("Waiting for list idle...\n");
		if ((result = rlListWaitForListIdle(handle, timeoutMs, &success, &doneId)) != rlSUCCESS) return result;
		printf("ListIdle: %d\n", doneId);

		(*pendingCount)--;
	}

	// close the list causing its contents to be transferred to the card
	printf("Closing list: index=%d...\n", *listId);
	if ((result = rlListSet(handle, *listId, list, false, -1)) != rlSUCCESS) return result;

	// run the list
	printf("Executing list: index=%d...\n", *listId);
	if ((result = rlListExecute(handle, *listId)) != rlSUCCESS) return result;

	(*pendingCount)++;
	*listId = (*listId + 1) % 2;

	return result;
}


// helper function which displays the description of the last error
static int printLastError(void)
{
	char errBuffer[1024] = { 0 };
	rlGetLastError(errBuffer, sizeof(errBuffer) - 1);
	printf("%s", errBuffer);
	return rlERROR;
}
